import pygame, sys, time, math, os, serial
from pygame.locals import *

import robot
from gui_libs import slider
from gui_libs import quadenc
from gui_libs import base
from gui_libs import led
from gui_libs import lightsensor
from gui_libs import textbox
from gui_libs import accelerometer
from gui_libs import receiver
from gui_libs import irbeacon
from gui_libs import buildinfo
from gui_libs import irreceiver
from gui_libs import irtextbox
from gui_libs import gyro

BLACK = (0, 0, 0)
WHITE = (255, 255, 255)
RED = (255, 0, 0)
GREEN = (34, 139, 34)
BLUE = (0, 0, 255)
GREY = (215, 215, 230)


class PyRoboMain:
    def __init__(self, width, height, ipm):
        self.ipm = ipm  
        self.width = width
        self.height = height
        self.screen = pygame.display.set_mode((self.width, self.height), 0, 32)
        self.caption = pygame.display.set_caption('Engi128 Robot')
        self.objectList = []
        self.ROBOTSENDID = "-1"
        
    def MainLoop(self):
        self.screen.fill(WHITE)
        ##BASE
        baseRobot = base.Base(self.screen, GREY, (300, 300), 130)        
        self.addObject(baseRobot)
        ##BEACON
        beacon = irbeacon.IRBeacon(self.screen, (300, 250), 10, 1, self.ipm)
        self.addObject(beacon)
        ##LED
        redLED = led.LED(self.screen, (250, 340), 20, RED, 'r', 70, self.ipm)
        self.addObject(redLED)
        greenLED = led.LED(self.screen, (300, 300), 20, GREEN, 'g', 127, self.ipm)
        self.addObject(greenLED)
        blueLED = led.LED(self.screen, (350, 340), 20, BLUE, 'b', 25, self.ipm)
        self.addObject(blueLED)
        ##BUILDinfo
        build_info = buildinfo.BuildInfo(self.screen, 600, 300, self.ipm)
        self.addObject(build_info)
        ##LEDbutton
        redButton = led.LEDButton(self.screen, (250, 340), 7, RED, 'r', self.ipm)
        self.addObject(redButton)
        greenButton = led.LEDButton(self.screen, (300, 300), 7, GREEN, 'g', self.ipm)
        self.addObject(greenButton)
        blueButton = led.LEDButton(self.screen, (350, 340), 7, BLUE, 'b', self.ipm)
        self.addObject(blueButton)
        ##ENCODERS
        quadEnc_left = quadenc.QuadEnc(self.screen, 500, 150, 'l', self.ipm)
        self.addObject(quadEnc_left)
        quadEnc_right = quadenc.QuadEnc(self.screen, 500, 350, 'r', self.ipm)
        self.addObject(quadEnc_right)
        ##SLIDERS
        slideLeft = slider.Slide(self.screen, 0, -101, 100, 200, 60, "l", self.ipm)
        self.addObject(slideLeft)
        slideRight = slider.Slide(self.screen, 0, -101, 100, 200, 530, "r", self.ipm)
        self.addObject(slideRight)
        ##MOTOR SYNC
        syncButton = slider.Sync((50, 50), self.screen, slideLeft, slideRight, self.ipm)
        self.addObject(syncButton)
        ##MOTOR STOP
        stopButton = slider.Stop((175, 50), self.screen, slideLeft, slideRight, self.ipm)
        self.addObject(stopButton)
        ##LIGHT SENSORS
        frLightSense = lightsensor.LightSensor(self.screen, 440 , 80, "fr", self.ipm)
        self.addObject(frLightSense)
        flLightSense = lightsensor.LightSensor(self.screen, 160, 80, "fl", self.ipm)
        self.addObject(flLightSense)
        rLightSense = lightsensor.LightSensor(self.screen, 300, 460, "r", self.ipm)
        self.addObject(rLightSense)
        ##GYRO
        gyroSensor = gyro.Gyro(self.screen, 560, 400, 200 , -180, 0, self.ipm)
        self.addObject(gyroSensor)
        ##ACCELEROMETER
        accel = accelerometer.Accelerometer(self.screen, 600, 150, self.ipm)
        self.addObject(accel)
        ##INFRARED-TX
        irText = irtextbox.IRtextbox(self, self.screen, 295, 20, 90, 50,  "IR Transmit", 5, self.ipm)
        self.addObject(irText)
        rfText = textbox.Textbox(self, self.screen, 500, 50, 90, 200, "RF Transmit", 31, self.ipm)
        self.addObject(rfText)
        ##INFRARED RX
        ir_fl = irreceiver.IRreceiver(self.screen, 202, 154, 30, 0, 30, self.ipm, define=True)
        self.ir_fl = ir_fl
        self.addObject(ir_fl)
        ir_lf = irreceiver.IRreceiver(self.screen, 104, 251, 30, 1, 30, self.ipm)
        self.addObject(ir_lf)
        ir_lb = irreceiver.IRreceiver(self.screen, 104, 349, 30, 2, 30, self.ipm)
        self.addObject(ir_lb)
        ir_bl = irreceiver.IRreceiver(self.screen, 201, 426, 30, 3, 30, self.ipm)
        self.addObject(ir_bl)
        ir_br = irreceiver.IRreceiver(self.screen, 339, 426, 30, 4, 30, self.ipm)
        self.addObject(ir_br)
        ir_rb = irreceiver.IRreceiver(self.screen, 436, 349, 30, 5, 30, self.ipm)
        self.addObject(ir_rb)
        ir_rf = irreceiver.IRreceiver(self.screen, 436, 251, 30, 6, 30, self.ipm)
        self.addObject(ir_rf)        
        ir_fr = irreceiver.IRreceiver(self.screen, 339, 154, 30, 7, 30, self.ipm)
        self.addObject(ir_fr)
        ##RADIO-RX
        radio_re = receiver.Receiver(self.screen, 500, 20, 170, "Radio Msg:", 90, self.ipm)
        self.addObject(radio_re)

        ##TIMER
        pygame.time.set_timer(USEREVENT, 800)
        running = True
        ##EVENT LOOP
        while running:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    running = False
                if event.type == pygame.MOUSEBUTTONDOWN:
                    eventpos = pygame.mouse.get_pos()
                    for obj in self.objectList:
                        obj.mouse_click(eventpos)
                if event.type == pygame.MOUSEMOTION:
                    eventpos = pygame.mouse.get_pos()
                    for obj in self.objectList:
                        obj.mouse_move(eventpos)
                if event.type == pygame.MOUSEBUTTONUP:
                    for obj in self.objectList:
                        obj.mouse_up()
                if event.type == USEREVENT:
                    self.ir_fl.update_ir()

                    for obj in self.objectList:
                        obj.timer()
                if event.type == KEYDOWN:
                    inkey = event.key
                    for obj in self.objectList:
                        obj.typing(inkey) 
            for obj in self.objectList:
                obj.draw()
            pygame.display.flip()
        pygame.quit()
        sys.exit(0)
            
    def addObject(self,  obj):
        self.objectList.append(obj)

    def setSendID(self, ID):
        self.ROBOTSENDID = ID

    def getSendID(self):
        return self.ROBOTSENDID

##RUNCOM - Com Port selection
class RunCom:
    def __init__(self, width, height):
        self.width = width
        self.height = height
        pygame.init()
        pygame.font.init()
        self.screen = pygame.display.set_mode((self.width, self.height), 0, 32)
        self.caption = pygame.display.set_caption('Engi128 Com Input')
        self.input_rect = pygame.rect.Rect(50, 100, 300, 25)
        self.input_rect_filler = pygame.rect.Rect(52, 102, 296, 21)
        self.instructions = pygame.rect.Rect(102, 140, 160, 30)
        self.notice = pygame.rect.Rect(100, 80, 160, 25)
        self.basicFont = pygame.font.SysFont(None, 30)
        self.smallFont = pygame.font.SysFont(None, 20)

    def display(self, msg):
        pygame.draw.rect(self.screen, BLACK, self.input_rect)
        pygame.draw.rect(self.screen, WHITE, self.input_rect_filler)
        self.screen.blit(self.smallFont.render("Enter 0 to run without robot", True, BLACK), self.instructions)
        if len(msg) != 0:
            self.screen.blit(self.basicFont.render(msg, True, BLACK), self.input_rect_filler)
        pygame.display.flip()                         

    def get_key(self):
        running = True
        while running:
            event = pygame.event.poll()
            if event.type == KEYDOWN:
                return event.key
            if event.type == pygame.QUIT:
                running = False
        pygame.quit()
        sys.exit(0)

    def start(self, com_num):
        ipm = robot.init_ipm(com_num)
        time.sleep(1)
        ipm.eval_string("import rone")
        MainWindow = PyRoboMain(800, 600, ipm)
        MainWindow.MainLoop()

    def ask(self, question):
        self.screen.fill(WHITE)
        current_string = ["c", "o", "m"]
        self.display(question + ": " + "".join(current_string))
        while 1:
            inkey = self.get_key()
            if inkey == K_BACKSPACE:
                current_string = current_string[0:-1]
            elif inkey == K_RETURN:
                try:
                    com_num =  "".join(current_string)
                    self.start(com_num)
                except serial.serialutil.SerialException:
                    if "".join(current_string) == "0":
                        
                        MainWindow = PyRoboMain(800, 570, None)
                        MainWindow.MainLoop()
                    else:
                        current_string = []
                        self.screen.blit(self.smallFont.render("Invalid com, try again", True, BLACK), self.notice)
            elif inkey <= 127:
                pygame.draw.rect(self.screen, WHITE, self.notice)
                current_string.append(chr(inkey))
            self.display(question + ": " + "".join(current_string))


if __name__ == "__main__":
    print "at main"
    startWindow = RunCom(400, 270)
    if len(sys.argv) < 2:
        try:
            dev = robot.autodetect()
            startWindow.start(dev)
        except robot.AutodetectError:
            startWindow.ask("serial port")
    else:
        if sys.argv[1] == 'none':
            startWindow.ask("serial port")
        startWindow.start(sys.argv[1])

