import component
import pygame

from pygame.locals import *
import sys




BLACK = (0, 0, 0)
WHITE = (255, 255, 255)
RED = (255, 0, 0)
GREEN = (34, 139, 34)
BLUE = (0, 0, 255)
GREY = (215, 215, 230)
DARKGREY = (84, 84, 94)

class Accelerometer(component.Component):
    def __init__(self, screen, left, top, ipm):
        self.screen = screen
        self.top = top
        self.left = left
        self.ipm = ipm
        self.smallFont = pygame.font.SysFont(None, 20)

        self.z_rect = pygame.rect.Rect(self.left, self.top - 1, 20, 102)
        self.xy_rect = pygame.rect.Rect( (self.left + 24), self.top-1, 101, 101)

        self.x = 0
        self.y = 0
        self.z = 0

        self.z_axis_pos = self.top + 50 
       # self.z_line = pygame.rect.Rect(self.left, self.z_axis_pos-1, 20, 3)

        self.xy_pos = ( (self.left + 75), (self.top + 50)) 

        self.z_label_rect = pygame.rect.Rect(self.left, self.top+110, 60, 15)
        self.x_label_rect = pygame.rect.Rect(self.left+130, self.top+50, 60, 15)
        self.y_label_rect = pygame.rect.Rect(self.left+75, self.top + 110, 60, 15)
        
    def draw(self):
        ##background rectangles
        pygame.draw.rect(self.screen, GREY, self.z_rect)
        pygame.draw.rect(self.screen, GREY, self.xy_rect)
        ##axis lines:
        pygame.draw.line(self.screen, DARKGREY, (self.left, self.top + 50), (self.left + 125, self.top + 50), 1)
        pygame.draw.line (self.screen, DARKGREY, (self.left + 75, self.top), (self.left + 75, self.top + 100), 1)
        ##z line
        pygame.draw.rect(self.screen, RED, pygame.rect.Rect(self.left, self.z_axis_pos - 1, 20, 3))
        ##xy circle
        pygame.draw.circle(self.screen, RED, self.xy_pos, 3, 0)
        ##label rectangles
        pygame.draw.rect(self.screen, WHITE, self.z_label_rect)
        pygame.draw.rect(self.screen, WHITE, self.x_label_rect)
        pygame.draw.rect(self.screen, WHITE, self.y_label_rect)
        self.screen.blit(self.smallFont.render("X: " + str(self.x).rstrip(), True, BLACK, WHITE), self.x_label_rect)
        self.screen.blit(self.smallFont.render("Y: " + str(self.y).rstrip(), True, BLACK, WHITE), self.y_label_rect)
        self.screen.blit(self.smallFont.render("Z: " + str(self.z).rstrip(), True, BLACK, WHITE), self.z_label_rect)



    def timer(self):
        self.x = self.point_val('x')
        self.y = self.point_val('y')
        self.z = self.point_val('z')
        self.z_plot()
        self.xy_plot()

    def point_val(self, axis):
        if self.ipm != None:
            accel_read = self.ipm.eval_string("rone.accelerometer_get_value('" + axis + "')")
            return accel_read
        else:
            return 500
    def z_plot(self):  
        z_val = int(self.z)/25
        axis_pos = self.top + 50 - z_val 
        if axis_pos < self.top:
            self.z_axis_pos = self.top
        if axis_pos > (self.top + 101):
            self.z_axis_pos = self.top + 96
        else:
            self.z_axis_pos = axis_pos
        
    def xy_plot(self):
        x_val = int(self.x)/25
        y_val = int(self.y)/25
        x_pos = self.left + 75 + x_val
        y_pos = self.top + 50 - y_val
        if x_pos < (self.left + 24):
            x_pos = self.left + 22
        if x_pos > (self.left + 125):
            x_pos = self.left + 123
        if y_pos > (self.top + 101):
            y_pos = self.top + 98
        if y_pos < (self.top):
            y_pos = self.top + 2
            
        self.xy_pos = (x_pos, y_pos) #self.left + 75 + x_val), (self.top + 50 - y_val))
