import component                                        
import pygame
from pygame.locals import *
import sys


BLACK = (0, 0, 0)
WHITE = (255, 255, 255)
RED = (255, 0, 0)
GREEN = (34, 139, 34)
BLUE = (0, 0, 255)
GREY = (215, 215, 230)





class Slide(component.Component):
    '''
    Slider class, takes in-
    screen: which pygame window to draw onto
    initval: starting value for the sliders
    minval: minimum value for the sliders
    maxval: maximum value for the sliders
    top: y value of where to place slider on screen
    left: x value of where to place slider on screen
    wheel: which wheel's motor the slider controls (e.g. 'left', 'right')
    ipm
    '''
    def __init__(self, screen, initval, minval, maxval, top, left, wheel, ipm):
        self.screen = screen
        self.wheel = wheel
        
        self.init_val = initval
        self.min_val = minval
        self.max_val = maxval

        self.top = top
        self.left = left
        self.height = (self.max_val - self.min_val)
        self.width = 12

        self.radius = 6
        self.topcenter = ( (self.left + self.radius), self.top )
        self.bottomcenter = ( (self.left + self.radius), (self.top + self.height) )
        self.start_knob_y = ((self.top + (self.height/2)) + self.init_val)

        self.main_rect = pygame.rect.Rect(self.left, self.top, self.width, self.height)
        self.background_rect = pygame.rect.Rect( (self.left - 40), self.top, (self.width + 80), self.height)
        self.top_background_rect = pygame.rect.Rect( (self.left-40), (self.top-50), (self.width + 80), 51)
        self.bottom_background_rect = pygame.rect.Rect( (self.left-40), (self.top + 199), (self.width+80), 50)

        self.down_state = 0

        self.knob_x = self.left + (self.width/2)
        self.knob = Knob(self.screen, self.knob_x, self.start_knob_y)

        self.numdisp = NumDisp(str(initval), self.max_val, self.min_val, self.top, self.height, self.left)
        self.b = self.init_val

        self.sync = 0

        self.ipm = ipm


    def draw(self):
        '''
        draws the slider on the screen
        '''

        pygame.draw.rect(self.screen, GREY, (self.left, self.top, self.width, self.height))
        pygame.draw.circle(self.screen, GREY, self.topcenter, self.radius, 0)
        pygame.draw.circle(self.screen, GREY, self.bottomcenter, self.radius, 0)
        self.knob.x_pos = self.knob_x
        self.knob.draw(self.knob.y_pos)
        text = self.numdisp.text(self.numdisp.y_num)
        negtext = self.numdisp.text(self.numdisp.negstr)
        left = self.left + 20
        top = self.top + (self.max_val - self.min_val)

        textRect = pygame.Rect(left, top, 40, 20)
        negLeft = self.left + 14
            
        negRect = pygame.Rect(negLeft, top, 10, 20)

        pygame.draw.rect(self.screen, WHITE, (left+10, top, 30, 21))
        self.screen.blit(text, textRect)
        self.screen.blit(negtext, negRect)
        pygame.display.update()



    def mouse_click(self, eventpos):
        '''
        Checks to see if event position of the mouse click is within the slider's main rectangle.
        If it is, moves the knob to the y event position and finds the percentage at which to run the motors.
        Also sets slider's down_state to 1
        '''
            
        if self.main_rect.collidepoint(eventpos):
            i, j = eventpos[0], eventpos[1]
            self.down_state = 1
            self.knob.y_pos = j
            self.b = self.numdisp.findNum(j)
            motor_command = Slide.motorRunString(self, self.b)
            if self.ipm != None:
                self.ipm.eval_string(motor_command)
        
            
    def mouse_move(self, eventpos):
        '''
        Checks to see if even position of mouse motion is within slider's background rectangle
        If it is, moves the knob to the corresponding y event positions and finds the percentage
        at which to run the motors.
        
        '''
        if self.sync:
            pass
        else:
            
            if self.background_rect.collidepoint(eventpos):
                if self.down_state == 1:
                    i, j = eventpos[0], eventpos[1]
                    self.knob.y_pos = j
                    self.b = self.numdisp.findNum(j)
                    motor_command = self.motorRunString(self.b)
                    if self.ipm != None:
                        self.ipm.eval_string(motor_command)
            if self.top_background_rect.collidepoint(eventpos):
                if self.down_state == 1:
                    self.knob.y_pos = self.top
                    self.b = self.numdisp.findNum(self.top)
                    motor_command = self.motorRunString(self.b)
                    if self.ipm != None:
                        self.ipm.eval_string(motor_command)
            if self.bottom_background_rect.collidepoint(eventpos):
                if self.down_state == 1:
                    self.knob.y_pos = self.top+ 200
                    self.b = self.numdisp.findNum(self.top + 200)
                    motor_command = self.motorRunString(self.b)
                    if self.ipm != None:
                        self.ipm.eval_string(motor_command)

    def mouse_up(self):
        self.down_state =0

    def stop_motor(self):
        self.knob.y_pos = self.top + 100
        if self.ipm != None:
            self.ipm.eval_string("rone.motor_stop('" + self.wheel + "')")
        self.numdisp.findNum(self.top + 100)

        

    def motorRunString(slider, b):
        if b >= 99:
            b = 98
        if b <= -99:
            b = -98
        if slider.wheel == "l":
                return "rone.motor_set_pwm('l', " + str(b) + ")"

        if slider.wheel == "r":
                return "rone.motor_set_pwm('r', " + str(b) + ")"


class Stop(component.Component):
    def __init__(self, center, screen, slideA, slideB, ipm):
        self.screen = screen
        self.slideA = slideA
        self.slideB = slideB
        self.ipm = ipm
        self.color = GREY
        self.color_ON = RED
        self.color_OFF = GREY
        self.center = center
        self.label_rect = pygame.rect.Rect((center[0] - 30), (center[1] - 30), 30, 50)
    def draw(self):
        smallFont = pygame.font.SysFont(None, 20)
        label_text = smallFont.render("Stop Motors", True, BLACK, WHITE)
        self.screen.blit(label_text, self.label_rect)
        pygame.draw.circle(self.screen, self.color, self.center, 10)
    def mouse_click(self, eventpos):
        i, j = self.center
        x, y = eventpos
        if (i - 20) <x<(i + 20) and (j - 20) < y < (j + 20):
                self.color = self.color_ON
                self.slideA.stop_motor()
                self.slideB.stop_motor()

    def mouse_up(self):
        self.color = self.color_OFF

                
class Sync(component.Component):
    def __init__(self, center, screen, slideA, slideB, ipm):
        self.screen = screen
        self.slideA = slideA
        self.slideB = slideB
        self.color = GREY
        self.color_ON = BLUE
        self.color_OFF = GREY
        self.center = center
        self.label_rect = pygame.rect.Rect((center[0] - 30), (center[1] - 30), 30, 50)
        self.ipm = ipm
    def draw(self):
        smallFont = pygame.font.SysFont(None, 20)

        label_text = smallFont.render("Sync Motors", True, BLACK, WHITE)
        self.screen.blit(label_text, self.label_rect)
        pygame.draw.circle(self.screen, self.color,self.center, 10)
    def mouse_click(self, eventpos):
        i, j = self.center
        x, y = eventpos
        if (i - 20) <x<(i + 20) and (j - 20) < y < (j + 20):
            if self.color == self.color_OFF:
                
                self.color = self.color_ON
                self.slideA.sync = 1
                self.slideB.sync = 1
                   
            else:
                self.color = self.color_OFF
                self.slideA.sync = 0
                self.slideB.sync = 0

        if self.slideA.main_rect.collidepoint(eventpos) or self.slideB.main_rect.collidepoint(eventpos):
            if self.slideA.sync == 1 and self.slideB.sync == 1:
                self.slideA.down_state = 1
                self.slideB.down_state = 1
                self.slideA.knob.y_pos = y
                self.slideB.knob.y_pos = y
                self.slideA.b = self.slideA.numdisp.findNum(y)
                self.slideB.b = self.slideB.numdisp.findNum(y)
                motor_command = Slide.motorRunString(self.slideA, self.slideA.b)
                motor_command2 = Slide.motorRunString(self.slideB, self.slideB.b)
                if self.ipm != None:
                    self.ipm.eval_string(motor_command)
                    self.ipm.eval_string(motor_command2)
    def mouse_move(self, eventpos):
        if self.slideA.down_state ==1 and self.slideB.down_state == 1:
            i, j = eventpos
            if self.slideA.background_rect.collidepoint(eventpos) or self.slideB.background_rect.collidepoint(eventpos):
            

                self.slideA.knob.y_pos = j
                self.slideB.knob.y_pos = j
                self.slideA.b = self.slideA.numdisp.findNum(j)
                self.slideB.b = self.slideB.numdisp.findNum(j)

            
                
            if self.slideA.top_background_rect.collidepoint(eventpos) or self.slideB.top_background_rect.collidepoint(eventpos):

                self.slideA.knob.y_pos = self.slideA.top
                self.slideB.knob.y_pos = self.slideB.top
                self.slideA.b = self.slideA.numdisp.findNum(self.slideA.top)
                self.slideB.b = self.slideB.numdisp.findNum(self.slideB.top)
                
            if self.slideA.bottom_background_rect.collidepoint(eventpos) or self.slideB.bottom_background_rect.collidepoint(eventpos):
                self.slideA.knob.y_pos = self.slideA.top + 200
                self.slideB.knob.y_pos = self.slideB.top + 200
                self.slideA.b = self.slideA.numdisp.findNum(self.slideA.top + 200)
                self.slideB.b = self.slideB.numdisp.findNum(self.slideB.top + 200)
              

            if self.ipm != None:
                motor_command_A = Slide.motorRunString(self.slideA, self.slideA.b)
                motor_command_B = Slide.motorRunString(self.slideB, self.slideB.b)

                self.ipm.eval_string(motor_command_A)
                self.ipm.eval_string(motor_command_B)
    
        
        
class Knob(component.Component):
    def __init__(self, screen, x, y):
        self.color = RED
        self.x_pos = x
        self.y_pos = y
        self.radius = 5
        self.screen = screen
    def draw(self, y):
        pygame.draw.circle(self.screen, self.color, (self.x_pos, y), self.radius, 0)

class NumDisp(component.Component):
    def __init__(self, y_num, max_val, min_val, top, height, left):
        self.y_num = y_num
        self.max_val = max_val
        self.min_val = min_val
        self.top = top
        self.height = height
        self.left = left

        self.neg = False
        self.negstr = " "


    def text(self, y_num):
        basicFont = pygame.font.SysFont(None, 30) 

        text = basicFont.render(y_num, True, BLACK, WHITE)
        
        return text


    def findNum(self, j):
        if j == self.top :
            self.y_num = str(self.max_val)
            b = self.max_val
            return b
        if j == self.top + self.height:
            self.y_num = str(self.min_val)
            b = self.min_val
            return b
    
        else:
            a = j - self.top
            b = self.max_val - a
            if b >= 0:
                self.y_num = str(b)
                self.neg = False
                self.negative()
                return b
   
            else:
                self.neg = True
                self.negative()
                abs_b = abs(b)
 
                self.y_num = str(abs_b)
                return b


        
    def negtext(self, neg):
        basicFont = pygame.font.SysFont(None, 30) 

        negtext = basicFont.render(neg, True, BLACK, WHITE)
        return negtext
                
    def negative(self):
        if self.neg:
            self.negstr = "-"
        else:
            self.negstr = " "

        
